(define-module (ordo context)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-9)
  #:use-module (srfi srfi-26)
  #:export (make-context
            context?
            context-connection
            context-scratch-dir
            set-context-scratch-dir!
            add-context-triggers!
            get-context-triggers
            context-triggered?
            register-context-var!
            context-ref
            bind-context-vars))

(define-record-type <context>
  (make-context connection vars scratch-dir)
  context?
  (connection context-connection)
  (scratch-dir context-scratch-dir set-context-scratch-dir!)
  (vars context-vars set-context-vars!)
  (triggers context-triggers set-context-triggers!))

(define (context-ref ctx var-name)
  (let ((kv (assoc var-name (context-vars ctx))))
    (if kv
        (cdr kv)
        (error (format #f "failed to resolve context reference: ~a" var-name)))))

(define (add-context-triggers! ctx triggers)
  (when triggers
    (set-context-triggers! ctx
                           (apply lset-adjoin equal? (or (context-triggers ctx) '()) triggers))))

(define (context-triggered? ctx trigger)
  (find (lambda (t) (equal? t trigger)) (context-triggers ctx)))

(define (register-context-var! ctx var-name val)
  (set-context-vars! ctx (assoc-set! (context-vars ctx) var-name val)))

(define-syntax bind-context-vars
  (syntax-rules ()
    ((bind-context-vars (var-name ...) proc)
     (lambda (ctx)
       (let ((var-name (context-ref ctx (quote var-name))) ...)
         (proc ctx))))))
